import sys
import os.path as op
import logging
import h5pyd as h5py
if __name__ == "__main__":
    from config import Config
else:
    from .config import Config

#
# Create domain or update timestamp if domain already exist
#
 
cfg = Config()

def getFolder(domain):
    username = cfg["hs_username"]
    password = cfg["hs_password"]
    endpoint = cfg["hs_endpoint"]
    #print("getFolder", domain)
    dir = h5py.Folder(domain, endpoint=endpoint, username=username, password=password)
    return dir

def createFolder(domain):
    username = cfg["hs_username"]
    password = cfg["hs_password"]
    endpoint = cfg["hs_endpoint"]
    #print("getFolder", domain)
    dir = h5py.Folder(domain, mode='x', endpoint=endpoint, username=username, password=password)
    return dir

def getFile(domain):
    username = cfg["hs_username"]
    password = cfg["hs_password"]
    endpoint = cfg["hs_endpoint"]
    #print("getFile", domain)
    fh = h5py.File(domain, mode='r', endpoint=endpoint, username=username, password=password)
    return fh

def createFile(domain):
    #print("createFile", domain)
    username = cfg["hs_username"]
    password = cfg["hs_password"]
    endpoint = cfg["hs_endpoint"]
    fh = h5py.File(domain, mode='x', endpoint=endpoint, username=username, password=password)
    return fh


   
def touchDomain(domain):

    make_folder = False
    if domain[-1] == '/':
        make_folder = True
        domain = domain[:-1]

    # get handle to parent folder
    parent_domain = op.dirname(domain) 

    if len(parent_domain) < 2:
        sys.exit("can't create top-level domain")

    if not parent_domain.endswith('/'):
        parent_domain += '/'
    try:
        getFolder(parent_domain)
    except IOError as oe:
        #print("errno:", oe.errno)
        if oe.errno in (404, 410):   # Not Found
            sys.exit("Parent domain: {} not found".format(parent_domain))
        elif oe.errno == 401:  # Unauthorized
            sys.exit("Authorization failure")
        elif oe.errno == 403:  # Forbidden
            sys.exit("Not allowed")
        else:
            sys.exit("Unexpected error: {}".format(oe))
    
    hdomain = None
    try:
        hdomain = getFile(domain)
    except IOError as oe:
        #print("errno:", oe.errno)
        if oe.errno in (404, 410):   # Not Found
            pass  # domain not found
        else:
            sys.exit("Unexpected error: {}".format(oe))

    if hdomain:
        if not make_folder:
            try:
                r = hdomain['/']
                # create/update attribute to update lastModified timestamp of domain
                r.attrs["hstouch"] = 1
                hdomain.close()
            except IOError as oe:
                sys.exit("Got error updating domain: {}".format(oe))
        else:
            sys.exit("Can not update timestamp of folder object")
        hdomain.close()
    else:
        # create domain
        if not make_folder:
            try:
                fh = createFile(domain)
                if cfg["verbose"]:
                    print("domain created: {}, root id: {}".format(domain, fh.id.id))
                fh.close()
            except IOError as oe:
                sys.exit("Got error updating domain: {}".format(oe))
        else:
            # make folder
            try:
                fh = createFolder(domain + '/')
                if cfg["verbose"]:
                    print("folder created", domain + '/')
                fh.close()
            except IOError as oe:
                sys.exit("Got error updating domain: {}".format(oe))

#
# Usage
#
def printUsage():
    print("usage: python hstouch.py [-v] [-e endpoint] [-u username] [-p password] [--loglevel debug|info|warning|error] [--logfile <logfile>] domains")
    print("example: python hstouch.py -e http://data.hdfgroup.org:7253 /hdfgroup/data/test/emptydomain.h5")
    sys.exit()

#
# Main
#
def main():
    domains = []
    argn = 1
    loglevel = logging.ERROR
    logfname=None
    cfg["verbose"] = False

    while argn < len(sys.argv):
        arg = sys.argv[argn]
        val = None
        if len(sys.argv) > argn + 1:
            val = sys.argv[argn+1]
    
        if arg in ("-h", "--help"):
            printUsage()
        elif arg in ("-v", "--verbose"):
            cfg["verbose"] = True
            argn += 1
        elif arg == "--loglevel":
            val = val.upper()
            if val == "DEBUG":
                loglevel = logging.DEBUG
            elif val == "INFO":
                loglevel = logging.INFO
            elif val in ("WARN", "WARNING"):
                loglevel = logging.WARNING
            elif val == "ERROR":
                loglevel = logging.ERROR
            else:
                printUsage()  
            argn += 2
        elif arg in ("-e", "--endpoint"):
            cfg["hs_endpoint"] = sys.argv[argn+1]
            argn += 2
        elif arg in ("-u", "--username"):
            cfg["hs_username"] = sys.argv[argn+1]
            argn += 2
        elif arg in ("-p", "--password"):
            cfg["hs_password"] = sys.argv[argn+1]
            argn += 2
        elif arg[0] == '-':
            printUsage()
        else:
            domains.append(arg)
            argn += 1
 
    if len(domains) == 0:
        # need a domain
        printUsage()

    # setup logging
    logging.basicConfig(filename=logfname, format='%(asctime)s %(message)s', level=loglevel)
    logging.debug("set log_level to {}".format(loglevel))

    for domain in domains:
        if not domain.startswith('/'):
            sys.exit("domain: {} must start with a slash".format(domain))
       
        touchDomain(domain)

if __name__ == "__main__":
    main()
    
     





##############################################################################
# Copyright by The HDF Group.                                                #
# All rights reserved.                                                       #
#                                                                            #
# This file is part of H5Serv (HDF5 REST Server) Service, Libraries and      #
# Utilities.  The full HDF5 REST Server copyright notice, including          #
# terms governing use, modification, and redistribution, is contained in     #
# the file COPYING, which can be found at the root of the source code        #
# distribution tree.  If you do not have access to this file, you may        #
# request a copy from help@hdfgroup.org.                                     #
##############################################################################
import config
import numpy as np

if config.get("use_h5py"):
    import h5py
else:
    import h5pyd as h5py

from common import ut, TestCase


class TestCommittedType(TestCase):
    def test_createtype(self):
        filename = self.getFileName("committed_type")
        print("filename:", filename)
        f = h5py.File(filename, "w")
        # create a compound numpy type
        dt = np.dtype([('real', np.float), ('img', np.float)])
        f['complex_type'] = dt
        ctype = f['complex_type']
        self.assertEqual(ctype.dtype.name, dt.name)
        self.assertEqual(len(ctype.dtype), len(dt))
        ctype.attrs["attr1"] = "this is a named datatype"
        dset = f.create_dataset('complex_dset', (10,), dtype=f['complex_type'])
        f.close()

if __name__ == '__main__':
    ut.main()



##############################################################################
# Copyright by The HDF Group.                                                #
# All rights reserved.                                                       #
#                                                                            #
# This file is part of H5Serv (HDF5 REST Server) Service, Libraries and      #
# Utilities.  The full HDF5 REST Server copyright notice, including          #
# terms governing use, modification, and redistribution, is contained in     #
# the file COPYING, which can be found at the root of the source code        #
# distribution tree.  If you do not have access to this file, you may        #
# request a copy from help@hdfgroup.org.                                     #
##############################################################################

import numpy as np
import math

import config

if config.get("use_h5py"):
    import h5py
else:
    import h5pyd as h5py

from common import ut, TestCase


class TestSetItemDataset(TestCase):
    def test_set_all(self):
        filename = self.getFileName("dset_set_all")
        print("filename:", filename)
        f = h5py.File(filename, "w")

        extent = 10
        dset = f.create_dataset('dset', (extent, extent), dtype='f8')
        arr = np.random.rand(extent, extent)
        dset[()] = arr
        f.close()


if __name__ == '__main__':
    ut.main()



##############################################################################
# Copyright by The HDF Group.                                                #
# All rights reserved.                                                       #
#                                                                            #
# This file is part of H5Serv (HDF5 REST Server) Service, Libraries and      #
# Utilities.  The full HDF5 REST Server copyright notice, including          #
# terms governing use, modification, and redistribution, is contained in     #
# the file COPYING, which can be found at the root of the source code        #
# distribution tree.  If you do not have access to this file, you may        #
# request a copy from help@hdfgroup.org.                                     #
##############################################################################
import unittest
import logging
import numpy as np
from h5type import special_dtype
from h5type import check_dtype
from base import Reference
import h5type
import six
 

class H5TypeTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(H5TypeTest, self).__init__(*args, **kwargs)
        # main
        self.logger = logging.getLogger()
        self.logger.setLevel(logging.INFO)

    def testBaseIntegerTypeItem(self):
        dt = np.dtype('<i1')
        typeItem = h5type.getTypeItem(dt)
        self.assertEqual(typeItem['class'], 'H5T_INTEGER')
        self.assertEqual(typeItem['base'], 'H5T_STD_I8LE')
        typeItem = h5type.getTypeResponse(typeItem) # non-verbose format
        self.assertEqual(typeItem['class'], 'H5T_INTEGER')
        self.assertEqual(typeItem['base'], 'H5T_STD_I8LE')


    def testBaseFloatTypeItem(self):
        dt = np.dtype('<f8')
        typeItem = h5type.getTypeItem(dt)
        self.assertEqual(typeItem['class'], 'H5T_FLOAT')
        self.assertEqual(typeItem['base'], 'H5T_IEEE_F64LE')
        typeItem = h5type.getTypeResponse(typeItem) # non-verbose format
        self.assertEqual(typeItem['class'], 'H5T_FLOAT')
        self.assertEqual(typeItem['base'], 'H5T_IEEE_F64LE')

    def testBaseStringTypeItem(self):
        dt = np.dtype('S3')
        typeItem = h5type.getTypeItem(dt)
        self.assertEqual(typeItem['class'], 'H5T_STRING')
        self.assertEqual(typeItem['length'], 3)
        self.assertEqual(typeItem['strPad'], 'H5T_STR_NULLPAD')
        self.assertEqual(typeItem['charSet'], 'H5T_CSET_ASCII')

    def testBaseStringUTFTypeItem(self):
        dt = np.dtype('U3')
        try:
            typeItem = h5type.getTypeItem(dt)
            self.assertTrue(False)  # expected exception
        except TypeError:
            pass # expected

    def testBaseVLenAsciiTypeItem(self):
        dt = special_dtype(vlen=six.binary_type)
        typeItem = h5type.getTypeItem(dt)
        self.assertEqual(typeItem['class'], 'H5T_STRING')
        self.assertEqual(typeItem['length'], 'H5T_VARIABLE')
        self.assertEqual(typeItem['strPad'], 'H5T_STR_NULLTERM')
        self.assertEqual(typeItem['charSet'], 'H5T_CSET_ASCII')

    def testBaseVLenUnicodeTypeItem(self):
        dt = special_dtype(vlen=six.text_type)
        typeItem = h5type.getTypeItem(dt)
        self.assertEqual(typeItem['class'], 'H5T_STRING')
        self.assertEqual(typeItem['length'], 'H5T_VARIABLE')
        self.assertEqual(typeItem['strPad'], 'H5T_STR_NULLTERM')
        self.assertEqual(typeItem['charSet'], 'H5T_CSET_UTF8')

    def testBaseEnumTypeItem(self):
        mapping = {'RED': 0, 'GREEN': 1, 'BLUE': 2}
        dt = special_dtype(enum=(np.int8, mapping))
        typeItem = h5type.getTypeItem(dt)
        self.assertEqual(typeItem['class'], 'H5T_ENUM')
        baseItem = typeItem['base']
        self.assertEqual(baseItem['class'], 'H5T_INTEGER')
        self.assertEqual(baseItem['base'], 'H5T_STD_I8LE')
        self.assertTrue('mapping' in typeItem)
        self.assertEqual(typeItem['mapping']['GREEN'], 1)

    def testBaseArrayTypeItem(self):
        dt = np.dtype('(2,2)<int32')
        typeItem = h5type.getTypeItem(dt)
        self.assertEqual(typeItem['class'], 'H5T_ARRAY')
        baseItem = typeItem['base']
        self.assertEqual(baseItem['class'], 'H5T_INTEGER')
        self.assertEqual(baseItem['base'], 'H5T_STD_I32LE')

    def testCompoundArrayTypeItem(self):
        dt = np.dtype([('a', '<i1'), ('b', 'S1', (10,))])
        typeItem = h5type.getTypeItem(dt)
        self.assertEqual(typeItem['class'], 'H5T_COMPOUND')
        fields = typeItem['fields']
        field_a = fields[0]
        self.assertEqual(field_a['name'], 'a')
        field_a_type = field_a['type']
        self.assertEqual(field_a_type['class'], 'H5T_INTEGER')
        self.assertEqual(field_a_type['base'], 'H5T_STD_I8LE')
        field_b = fields[1]
        self.assertEqual(field_b['name'], 'b')
        field_b_type = field_b['type']
        self.assertEqual(field_b_type['class'], 'H5T_ARRAY')
        self.assertEqual(field_b_type['dims'], (10,))
        field_b_basetype = field_b_type['base']
        self.assertEqual(field_b_basetype['class'], 'H5T_STRING')


    def testOpaqueTypeItem(self):
        dt = np.dtype('V200')
        typeItem = h5type.getTypeItem(dt)
        self.assertEqual(typeItem['class'], 'H5T_OPAQUE')
        self.assertTrue('base' not in typeItem)

    def testVlenDataItem(self):
        dt = special_dtype(vlen=np.dtype('int32'))
        typeItem = h5type.getTypeItem(dt)
        self.assertEqual(typeItem['class'], 'H5T_VLEN')
        self.assertEqual(typeItem['size'], 'H5T_VARIABLE')
        baseItem = typeItem['base']
        self.assertEqual(baseItem['base'], 'H5T_STD_I32LE')

    def testCompoundTypeItem(self):
        dt = np.dtype([("temp", np.float32), ("pressure", np.float32), ("wind", np.int16)])
        typeItem = h5type.getTypeItem(dt)
        self.assertEqual(typeItem['class'], 'H5T_COMPOUND')
        self.assertTrue('fields' in typeItem)
        fields = typeItem['fields']
        self.assertEqual(len(fields), 3)
        tempField = fields[0]
        self.assertEqual(tempField['name'], 'temp')
        self.assertTrue('type' in tempField)
        tempFieldType = tempField['type']
        self.assertEqual(tempFieldType['class'], 'H5T_FLOAT')
        self.assertEqual(tempFieldType['base'], 'H5T_IEEE_F32LE')

        typeItem = h5type.getTypeResponse(typeItem) # non-verbose format
        self.assertEqual(typeItem['class'], 'H5T_COMPOUND')
        self.assertTrue('fields' in typeItem)
        fields = typeItem['fields']
        self.assertEqual(len(fields), 3)
        tempField = fields[0]
        self.assertEqual(tempField['name'], 'temp')
        self.assertTrue('type' in tempField)
        tempFieldType = tempField['type']
        self.assertEqual(tempFieldType['class'], 'H5T_FLOAT')
        self.assertEqual(tempFieldType['base'], 'H5T_IEEE_F32LE')

    def testCreateBaseType(self):
        dt = h5type.createDataType('H5T_STD_U32BE')
        self.assertEqual(dt.name, 'uint32')
        self.assertEqual(dt.byteorder, '>')
        self.assertEqual(dt.kind, 'u')

        dt = h5type.createDataType('H5T_STD_I16LE')
        self.assertEqual(dt.name, 'int16')
        self.assertEqual(dt.kind, 'i')

        dt = h5type.createDataType('H5T_IEEE_F64LE')
        self.assertEqual(dt.name, 'float64')
        self.assertEqual(dt.kind, 'f')

        dt = h5type.createDataType('H5T_IEEE_F32LE')
        self.assertEqual(dt.name, 'float32')
        self.assertEqual(dt.kind, 'f')

        typeItem = { 'class': 'H5T_INTEGER', 'base': 'H5T_STD_I32BE' }
        dt = h5type.createDataType(typeItem)
        self.assertEqual(dt.name, 'int32')
        self.assertEqual(dt.kind, 'i')

    def testCreateBaseStringType(self):
        typeItem = { 'class': 'H5T_STRING', 'charSet': 'H5T_CSET_ASCII', 'length': 6 }
        dt = h5type.createDataType(typeItem)
        if six.PY3:
            self.assertEqual(dt.name, 'bytes48')
        else:
            self.assertEqual(dt.name, 'string48')
        self.assertEqual(dt.kind, 'S')

    def testCreateBaseUnicodeType(self):
        typeItem = { 'class': 'H5T_STRING', 'charSet': 'H5T_CSET_UTF8', 'length': 32 }
        try:
            dt = h5type.createDataType(typeItem)
            self.assertTrue(False)  # expected exception
        except TypeError:
            pass

    def testCreateNullTermStringType(self):
        typeItem = { 'class': 'H5T_STRING', 'charSet': 'H5T_CSET_ASCII',
            'length': 6, 'strPad': 'H5T_STR_NULLTERM'}
        dt = h5type.createDataType(typeItem)
        if six.PY3:
            self.assertEqual(dt.name, 'bytes48')
        else:
            self.assertEqual(dt.name, 'string48')
        self.assertEqual(dt.kind, 'S')


    def testCreateVLenStringType(self):
        typeItem = { 'class': 'H5T_STRING', 'charSet': 'H5T_CSET_ASCII', 'length': 'H5T_VARIABLE' }
        dt = h5type.createDataType(typeItem)
        self.assertEqual(dt.name, 'object')
        self.assertEqual(dt.kind, 'O')
        self.assertEqual(check_dtype(vlen=dt), bytes)
        

    def testCreateVLenUTF8Type(self):
        typeItem = { 'class': 'H5T_STRING', 'charSet': 'H5T_CSET_UTF8', 'length': 'H5T_VARIABLE' }
        dt = h5type.createDataType(typeItem)
        self.assertEqual(dt.name, 'object')
        self.assertEqual(dt.kind, 'O')
        self.assertEqual(check_dtype(vlen=dt), six.text_type)

    def testCreateVLenDataType(self):
        typeItem = {'class': 'H5T_VLEN', 'base': 'H5T_STD_I32BE'}
        dt = h5type.createDataType(typeItem)
        self.assertEqual(dt.name, 'object')
        self.assertEqual(dt.kind, 'O')

    def testCreateOpaqueType(self):
        typeItem = {'class': 'H5T_OPAQUE', 'size': 200}
        dt = h5type.createDataType(typeItem)
        self.assertEqual(dt.name, 'void1600')
        self.assertEqual(dt.kind, 'V')

    def testCreateCompoundType(self):
        typeItem = {
            'class': 'H5T_COMPOUND', 'fields':
                [{'name': 'temp',     'type': 'H5T_IEEE_F32LE'},
                 {'name': 'pressure', 'type': 'H5T_IEEE_F32LE'},
                 {'name': 'location', 'type': {
                     'length': 'H5T_VARIABLE',
                     'charSet': 'H5T_CSET_ASCII',
                     'class': 'H5T_STRING',
                     'strPad': 'H5T_STR_NULLTERM'}},
                 {'name': 'wind',     'type': 'H5T_STD_I16LE'}]
        }

        dt = h5type.createDataType(typeItem)
        self.assertEqual(dt.name, 'void144')
        self.assertEqual(dt.kind, 'V')
        self.assertEqual(len(dt.fields), 4)
        dtLocation = dt[2]
        self.assertEqual(dtLocation.name, 'object')
        self.assertEqual(dtLocation.kind, 'O')
        self.assertEqual(check_dtype(vlen=dtLocation), bytes)

    def testCreateCompoundTypeUnicodeFields(self):
        typeItem = {
            'class': 'H5T_COMPOUND', 'fields':
                [{'name': u'temp',     'type': 'H5T_IEEE_F32LE'},
                 {'name': u'pressure', 'type': 'H5T_IEEE_F32LE'},
                 {'name': u'wind',     'type': 'H5T_STD_I16LE'}]
        }

        dt = h5type.createDataType(typeItem)   
        self.assertEqual(dt.name, 'void80')
        self.assertEqual(dt.kind, 'V')
        self.assertEqual(len(dt.fields), 3)

    def testCreateArrayType(self):
        typeItem = {'class': 'H5T_ARRAY',
                    'base': 'H5T_STD_I64LE',
                    'dims': (3, 5) }
        dt = h5type.createDataType(typeItem)
        self.assertEqual(dt.name, 'void960')
        self.assertEqual(dt.kind, 'V')

    def testCreateArrayIntegerType(self):
        typeItem = {'class': 'H5T_INTEGER',
                    'base': 'H5T_STD_I64LE',
                    'dims': (3, 5) }
        dt = h5type.createDataType(typeItem)
        self.assertEqual(dt.name, 'void960')
        self.assertEqual(dt.kind, 'V')

    def testCreateCompoundArrayType(self):
        typeItem = {
            "class": "H5T_COMPOUND",
            "fields": [
                {
                    "type": {
                        "base": "H5T_STD_I8LE",
                        "class": "H5T_INTEGER"
                    },
                    "name": "a"
                },
                {
                    "type": {
                        "dims": [
                            10
                        ],
                        "base": {
                            "length": 1,
                            "charSet": "H5T_CSET_ASCII",
                            "class": "H5T_STRING",
                            "strPad": "H5T_STR_NULLPAD"
                        },
                    "class": "H5T_ARRAY"
                },
                "name": "b"
                }
            ]
        }
        dt = h5type.createDataType(typeItem)   
        self.assertEqual(len(dt.fields), 2)
        self.assertTrue('a' in dt.fields.keys())
        self.assertTrue('b' in dt.fields.keys())
        
    def testRefType(self):
        # todo - special_dtype not implemented
        dt = special_dtype(ref=Reference)
        self.assertEqual(dt.kind, 'S')
        self.assertTrue(dt.metadata['ref'] is Reference)
            
        reftype = check_dtype(ref=dt)
        self.assertTrue(reftype is Reference)


if __name__ == '__main__':
    #setup test files

    unittest.main()



import sys
import os.path as op
import logging
import h5pyd as h5py
if __name__ == "__main__":
    from config import Config
else:
    from .config import Config

#
# Delete domain
#
 
cfg = Config()
 
def getFolder(domain, mode='r'):
    username = cfg["hs_username"]
    password = cfg["hs_password"]
    endpoint = cfg["hs_endpoint"]
    dir = h5py.Folder(domain, mode=mode, endpoint=endpoint, username=username, password=password)
    return dir
 
   
def deleteDomain(domain):

    # get handle to parent folder
    if domain.endswith('/'):
        path = domain[:-1]
    else:
        path = domain
    parent_domain = op.dirname(path)
    base_name = op.basename(path)

    if len(parent_domain) < 2:
        sys.exit("can't get parent domain")

    if not parent_domain.endswith('/'):
        parent_domain += '/'
    try:
        hparent = getFolder(parent_domain, mode='a')
    except IOError as oe:
        if oe.errno == 404:   # Not Found
            sys.exit("Parent domain: {} not found".format(parent_domain))
        elif oe.errno == 401:  # Unauthorized
            sys.exit("Authorization failure")
        elif oe.errno == 403:  # Forbidden
            sys.exit("Not allowed")
        else:
            sys.exit("Unexpected error: {}".format(oe))
    
    if base_name not in hparent:
        # note - this may happen if the domain was recently created and not 
        # yet synced to S3
        sys.exit("domain: {} not found".format(domain))

    # delete the domain
    del hparent[base_name]
    if cfg["verbose"]:
        if domain.endswith('/'):
            print("Folder: {} deleted".format(domain))
        else:
            print("Domain: {} deleted".format(domain))
      
#
# Usage
#
def printUsage():
    print("usage: python hsdel.py [-v] [-e endpoint] [-u username] [-p password] [--loglevel debug|info|warning|error] [--logfile <logfile>] domains")
    print("example: python hsdel.py -e http://data.hdfgroup.org:7253 /hdfgroup/data/test/deleteme.h5")
    sys.exit()

#
# Main
#
def main():
    domains = []
    argn = 1
    loglevel = logging.ERROR
    logfname=None
    cfg["verbose"] = False

    while argn < len(sys.argv):
        arg = sys.argv[argn]
        val = None
        if len(sys.argv) > argn + 1:
            val = sys.argv[argn+1]
    
        if arg in ("-h", "--help"):
            printUsage()
        elif arg in ("-e", "--endpoint"):
            cfg["hs_endpoint"] = val
            argn += 2
        elif arg in ("-u", "--username"):
            cfg["hs_username"] = val
            argn += 2
        elif arg in ("-p", "--password"):
            cfg["hs_password"] = val
            argn += 2
        elif arg in ("-v", "--verbose"):
            cfg["verbose"] = True
            argn += 1
        elif arg == "--loglevel":
            val = val.upper()
            if val == "DEBUG":
                loglevel = logging.DEBUG
            elif val == "INFO":
                loglevel = logging.INFO
            elif val in ("WARN", "WARNING"):
                loglevel = logging.WARNING
            elif val == "ERROR":
                loglevel = logging.ERROR
            else:
                printUsage()  
            argn += 2
        elif arg == '--logfile':
            logfname = val
            argn += 2
        elif arg[0] == '-':
            printUsage()
        else:
            domains.append(arg)
            argn += 1
 
    if len(domains) == 0:
        # need a domain
        printUsage()


    logging.basicConfig(filename=logfname, format='%(asctime)s %(message)s', level=loglevel)
    logging.debug("set log_level to {}".format(loglevel))

    for domain in domains:
        if not domain.startswith('/'):
            sys.exit("domain: {} must start with a slash".format(domain))
         
        deleteDomain(domain)


if __name__ == "__main__":
    main()


##############################################################################
# Copyright by The HDF Group.                                                #
# All rights reserved.                                                       #
#                                                                            #
# This file is part of HSDS (HDF5 Scalable Data Service), Libraries and      #
# Utilities.  The full HSDS copyright notice, including                      #
# terms governing use, modification, and redistribution, is contained in     #
# the file COPYING, which can be found at the root of the source code        #
# distribution tree.  If you do not have access to this file, you may        #
# request a copy from help@hdfgroup.org.                                     #
##############################################################################

import sys
import logging
    
try:
    import h5py 
    import h5pyd 
except ImportError as e:
    sys.stderr.write("ERROR : %s : install it to use this utility...\n" % str(e)) 
    sys.exit(1)
 
if __name__ == "__main__":
    from config import Config
    from utillib import load_file
else:
    from .config import Config
    from .utillib import load_file

__version__ = '0.0.1'

UTILNAME = 'hsget'

#----------------------------------------------------------------------------------
def usage():
    print("Usage:\n")
    print(("    %s [ OPTIONS ]  DOMAIN DES" % UTILNAME))
    print("")
    print("Description:")
    print("    Copy server domain to local HDF5 file")
    print("       DOMAIN: HDF Server domain (Unix or DNS style)")
    print("       DES: HDF5 file to be created ")
    print("")
    print("Options:")
    print("     -v | --verbose :: verbose output")
    print("     -e | --endpoint <domain> :: The HDF Server endpoint, e.g. http://example.com:8080")
    print("     -u | --user <username>   :: User name credential")
    print("     -p | --password <password> :: Password credential")
    print("     -c | --conf <file.cnf>  :: A credential and config file")
    print("     --cnf-eg        :: Print a config file and then exit")
    print("     --logfile <logfile> :: logfile path")
    print("     --loglevel debug|info|warning|error :: Change log level")
    print("     --nodata :: Do not download dataset data")
    print("     -4 :: Force ipv4 for any file staging (doesn\'t set hsds loading net)")
    print("     -6 :: Force ipv6 (see -4)")
    print("     -h | --help    :: This message.")
    print("")
    print(("%s version %s\n" % (UTILNAME, __version__)))
#end print_usage

#----------------------------------------------------------------------------------
def print_config_example():
    print("# default")
    print("hs_username = <username>")
    print("hs_password = <passwd>")
    print("hs_endpoint = https://hdfgroup.org:7258")
#print_config_example

#----------------------------------------------------------------------------------
def main():
     
    loglevel = logging.ERROR
    verbose = False
    nodata = False
    cfg = Config()  #  config object
    endpoint=cfg["hs_endpoint"]
    username=cfg["hs_username"]
    password=cfg["hs_password"]
    logfname=None
    ipvfam=None
    
    des_file = None
    src_domain = None
    argn = 1
    while argn < len(sys.argv):
        arg = sys.argv[argn]
        val = None
         
        if arg[0] == '-' and src_domain is not None:
            # options must be placed before filenames
            print("options must precead source files")
            usage()
            sys.exit(-1)
        if len(sys.argv) > argn + 1:
            val = sys.argv[argn+1] 
        if arg in ("-v", "--verbose"):
            verbose = True
            argn += 1
        elif arg == "--nodata":
            nodata = True
            argn += 1
        elif arg == "--loglevel":
            if val == "debug":
                loglevel = logging.DEBUG
            elif val == "info":
                loglevel = logging.INFO
            elif val == "warning":
                loglevel = logging.WARNING
            elif val == "error":
                loglevel = logging.ERROR
            else:
                print("unknown loglevel")
                usage()  
                sys.exit(-1)
            argn += 2
        elif arg == '--logfile':
            logfname = val
            argn += 2     
        elif arg == '-4':
            ipvfam = 4
        elif arg == '-6':
            ipvfam = 6
        elif arg in ("-h", "--help"):
            usage()
            sys.exit(0)
        elif arg in ("-e", "--endpoint"):
            endpoint = val
            argn += 2
        elif arg in ("-u", "--username"):
            username = val
            argn += 2
        elif arg in ("-p", "--password"):
            password = val
            argn += 2
        elif arg == '--cnf-eg':
            print_config_example()
            sys.exit(0)
        elif arg[0] == '-':
            usage()
            sys.exit(-1)
        elif src_domain is None:
            src_domain = arg
            argn += 1
        elif des_file is None:
            des_file = arg
            argn += 1
        else:
            usage()
            sys.exit(-1)
             
    # setup logging
    logging.basicConfig(filename=logfname, format='%(asctime)s %(message)s', level=loglevel)
    logging.debug("set log_level to {}".format(loglevel))
    
    # end arg parsing
    logging.info("username: {}".format(username))
    logging.info("password: {}".format(password))
    logging.info("endpoint: {}".format(endpoint))
    logging.info("verbose: {}".format(verbose))
    
    if src_domain is None or des_file is None:
        # need at least a src and destination
        usage()
        sys.exit(-1)
     
    logging.info("source domain: {}".format(src_domain))
    logging.info("target file: {}".format(des_file))
     
        
    if endpoint is None:
        logging.error('No endpoint given, try -h for help\n')
        sys.exit(1)
    logging.info("endpoint: {}".format(endpoint))

    # get a handle to input domain
    try:
        fin = h5pyd.File(src_domain, mode='r', endpoint=endpoint, username=username, password=password, use_cache=True)
    except IOError as ioe:
        if ioe.errno == 403:
            logging.error("No read access to domain: {}".format(src_domain))
        elif ioe.errno == 404:
            logging.error("Domain: {} not found".format(src_domain))
        elif ioe.errno == 410:
            logging.error("Domain: {} has been recently deleted".format(src_domain))
        else:
            logging.error("Error opening domain {}: {}".format(src_domain, ioe))
        sys.exit(1)

    # create the output HDF5 file
    try:
        fout = h5py.File(des_file, 'w')
    except IOError as ioe:
        logging.error("Error creating file {}: {}".format(des_file, ioe))
        sys.exit(1)

    try: 
        load_file(fin, fout, verbose=verbose, nodata=nodata)  
        msg = "Domain {} downloaded to file: {}".format(src_domain, des_file)
        logging.info(msg)
        if verbose:
            print(msg)    
    except KeyboardInterrupt:
        logging.error('Aborted by user via keyboard interrupt.')
        sys.exit(1)
#__main__
if __name__ == "__main__":
    main()




##############################################################################
# Copyright by The HDF Group.                                                #
# All rights reserved.                                                       #
#                                                                            #
# This file is part of H5Serv (HDF5 REST Server) Service, Libraries and      #
# Utilities.  The full HDF5 REST Server copyright notice, including          #
# terms governing use, modification, and redistribution, is contained in     #
# the file COPYING, which can be found at the root of the source code        #
# distribution tree.  If you do not have access to this file, you may        #
# request a copy from help@hdfgroup.org.                                     #
##############################################################################

"""
    Implements support for HDF5 compression filters via the high-level
    interface.  The following types of filter are available:

    "gzip"
        Standard DEFLATE-based compression, at integer levels from 0 to 9.
        Built-in to all public versions of HDF5.  Use this if you want a
        decent-to-good ratio, good portability, and don't mind waiting.

    "lzf"
        Custom compression filter for h5py.  This filter is much, much faster
        than gzip (roughly 10x in compression vs. gzip level 4, and 3x faster
        in decompressing), but at the cost of a worse compression ratio.  Use
        this if you want cheap compression and portability is not a concern.

    "szip"
        Access to the HDF5 SZIP encoder.  SZIP is a non-mainstream compression
        format used in space science on integer and float datasets.  SZIP is
        subject to license requirements, which means the encoder is not
        guaranteed to be always available.  However, it is also much faster
        than gzip.

    The following constants in this module are also useful:

    decode
        Tuple of available filter names for decoding

    encode
        Tuple of available filter names for encoding
"""

from __future__ import absolute_import, division

import numpy as np
# from .. import h5z, h5p, h5d


_COMP_FILTERS = {'gzip': 'H5Z_FILTER_DEFLATE',
                'szip': 'H5Z_FILTER_SZIP',
                'lzf': 'H5Z_FILTER_LZF',
                'shuffle': 'H5Z_FILTER_SHUFFLE',
                'fletcher32': 'H5Z_FILTER_FLETCHER32',
                'scaleoffset': 'H5Z_FILTER_SCALEOFFSET' }

DEFAULT_GZIP = 4
DEFAULT_SZIP = ('nn', 8)
SO_INT_MINBITS_DEFAULT = 0

def _gen_filter_tuples():
    """ Bootstrap function to figure out what filters are available. """
    dec = []
    enc = []
    for name, code in _COMP_FILTERS.items():
        # TBD: Provide a REST operation to query which filters are available
        #info = h5z.get_filter_info(code)
        enc.append(name)
        dec.append(name)

    return tuple(dec), tuple(enc)



decode, encode = _gen_filter_tuples()

def generate_dcpl(shape, dtype, chunks, compression, compression_opts,
                  shuffle, fletcher32, maxshape, scaleoffset):
    """ Generate a dataset creation property list.

    Undocumented and subject to change without warning.
    """

    plist = {}

    if shape == ():
        if any((chunks, compression, compression_opts, shuffle, fletcher32,
                scaleoffset is not None)):
            raise TypeError("Scalar datasets don't support chunk/filter options")
        if maxshape and maxshape != ():
            raise TypeError("Scalar datasets cannot be extended")
        return plist

    def rq_tuple(tpl, name):
        """ Check if chunks/maxshape match dataset rank """
        if tpl in (None, True):
            return
        try:
            tpl = tuple(tpl)
        except TypeError:
            raise TypeError('"%s" argument must be None or a sequence object' % name)
        if len(tpl) != len(shape):
            raise ValueError('"%s" must have same rank as dataset shape' % name)

    rq_tuple(chunks, 'chunks')
    rq_tuple(maxshape, 'maxshape')

    if compression is not None:

        if compression not in encode and not isinstance(compression, int):
            raise ValueError('Compression filter "%s" is unavailable' % compression)

        if compression == 'gzip':
            if compression_opts is None:
                gzip_level = DEFAULT_GZIP
            elif compression_opts in range(10):
                gzip_level = compression_opts
            else:
                raise ValueError("GZIP setting must be an integer from 0-9, not %r" % compression_opts)

        elif compression == 'lzf':
            if compression_opts is not None:
                raise ValueError("LZF compression filter accepts no options")

        elif compression == 'szip':
            if compression_opts is None:
                compression_opts = DEFAULT_SZIP

            err = "SZIP options must be a 2-tuple ('ec'|'nn', even integer 0-32)"
            try:
                szmethod, szpix = compression_opts
            except TypeError:
                raise TypeError(err)
            if szmethod not in ('ec', 'nn'):
                raise ValueError(err)
            if not (0<szpix<=32 and szpix%2 == 0):
                raise ValueError(err)

    elif compression_opts is not None:
        # Can't specify just compression_opts by itself.
        raise TypeError("Compression method must be specified")

    if scaleoffset is not None:
        # scaleoffset must be an integer when it is not None or False,
        # except for integral data, for which scaleoffset == True is
        # permissible (will use SO_INT_MINBITS_DEFAULT)

        if scaleoffset < 0:
            raise ValueError('scale factor must be >= 0')

        if dtype.kind == 'f':
            if scaleoffset is True:
                raise ValueError('integer scaleoffset must be provided for '
                                 'floating point types')
        elif dtype.kind in ('u', 'i'):
            if scaleoffset is True:
                scaleoffset = SO_INT_MINBITS_DEFAULT
        else:
            raise TypeError('scale/offset filter only supported for integer '
                            'and floating-point types')

        # Scale/offset following fletcher32 in the filter chain will (almost?)
        # always triggera a read error, as most scale/offset settings are
        # lossy. Since fletcher32 must come first (see comment below) we
        # simply prohibit the combination of fletcher32 and scale/offset.
        if fletcher32:
            raise ValueError('fletcher32 cannot be used with potentially lossy'
                             ' scale/offset filter')
    # End argument validation

    if (chunks is True) or \
    (chunks is None and any((shuffle, fletcher32, compression, maxshape,
                             scaleoffset is not None))):
        chunks = guess_chunk(shape, maxshape, dtype.itemsize)

    if maxshape is True:
        maxshape = (None,)*len(shape)

    if chunks is not None:
        #plist.set_chunk(chunks)
        # set layout key
        layout = { 'class': 'H5D_CHUNKED'}
        layout['dims'] = chunks
        plist['layout'] = layout
        plist['fillTime'] = 'H5D_FILL_TIME_ALLOC'  # prevent resize glitch

    filters = []
    # MUST be first, to prevent 1.6/1.8 compatibility glitch
    if fletcher32:
        filter_fletcher32 = { 'class': 'H5Z_FLETCHER_DEFLATE' }
        filter_fletcher32['id'] = 3
        filters.append(filter_fletcher32)

    # scale-offset must come before shuffle and compression
    if scaleoffset is not None:
        filter_scaleoffset = { 'class': 'H5Z_FILTER_SCALEOFFSET' }
        filter_scaleoffset['id'] = 6
        filter_scaleoffset['scaleOffset'] = scaleoffset
        if dtype.kind in ('u', 'i'):
            #plist.set_scaleoffset(h5z.SO_INT, scaleoffset)
            filter_scaleoffset['scaleType'] = 'H5Z_SO_INT'
        else: # dtype.kind == 'f'
            #plist.set_scaleoffset(h5z.SO_FLOAT_DSCALE, scaleoffset)
            filter_scaleoffset['scaleType'] = 'H5Z_SO_FLOAT_DSCALE'
        filters.append(filter_scaleoffset)

    if shuffle:
        filter_shuffle = { 'class': 'H5Z_FILTER_SHUFFLE' }
        filter_shuffle['id'] = 2
        filters.append(filter_shuffle)

    if compression == 'gzip':
        #plist.set_deflate(gzip_level)
        filter_gzip = { 'class': 'H5Z_FILTER_DEFLATE' }
        filter_gzip['id'] = 1
        filter_gzip['level'] = gzip_level
        filters.append(filter_gzip)
    elif compression == 'lzf':
        #plist.set_filter(h5z.FILTER_LZF, h5z.FLAG_OPTIONAL)
        filter_lzf = { 'class': 'H5Z_FILTER_LZF' }
        filter_lzf['id'] = 32000
        filters.append(filter_lzf)

    elif compression == 'szip':
        opts = {'ec': 'H5Z_SZIP_EC_OPTION_MASK', 'nn': 'H5Z_SZIP_NN_OPTION_MASK' }
        # plist.set_szip(opts[szmethod], szpix)
        filter_szip = { 'class': 'H5Z_FILTER_SZIP' }
        filter_szip['id'] = 4
        filter_szip['coding'] = opts
        if szmethod == 'ec':
            filter_szip['coding'] = 'H5_SZIP_EC_OPTION_MASK'
        else:   # 'nn'
            filter_szip['coding'] = 'H5_SZIP_NN_OPTION_MASK'
        filter_szip['bitsPerPixel'] = szpix
        filters.append(filter_szip)

    elif isinstance(compression, int):
        # TBD - don't have a way to query available filters via REST API
        # just throw ValueError for now
        raise ValueError("Unsupported compression filter: {}".format(compression))
        """
        if not h5z.filter_avail(compression):
            raise ValueError("Unknown compression filter number: %s" % compression)
        filter_ext = { 'id': compression }
        for k in compression_opts:
            filter_ext[k] = compression_opts[k]
        filters.append(filter_ext)
        plist.set_filter(compression, h5z.FLAG_OPTIONAL, compression_opts)
        """

    if len(filters) > 0:
        plist["filters"] = filters

    return plist

def get_filters(plist):
    """ Extract a dictionary of active filters from a DCPL, along with
    their settings.

    Undocumented and subject to change without warning.
    """

    filter_names = {'H5Z_FILTER_DEFLATE': 'gzip',
               'H5Z_FILTER_SZIP': 'szip',
               'H5Z_FILTER_SHUFFLE': 'shuffle',
               'H5Z_FILTER_FLETCHER32': 'fletcher32',
               'H5Z_FILTER_LZF': 'lzf',
               'H5Z_FILTER_SCALEOFFSET': 'scaleoffset' }

    vals = None
    pipeline = {}
    if 'filters' not in plist:
        return pipeline


    filters = plist['filters']

    for filter in filters:

        if filter['class'] == 'H5Z_FILTER_DEFLATE':
            vals = filter['level'] # gzip level

        elif filter['class'] == 'H5Z_FILTER_SZIP':
            mask = None
            if filter['coding'] == "H5Z_SZIP_EC_OPTION_MASK":
                mask = 'ec'
            elif filter['coding'] == "H5Z_SZIP_NN_OPTION_MASK":
                mask = 'nn'
            else:
                raise TypeError("Unknown SZIP configuration")
            pixels = filter['bitsPerPixel']
            vals = (mask, pixels)
        elif filter['class'] == 'H5Z_FILTER_LZF':
            vals = None
        else:
            if vals and len(vals) == 0:
               vals = None
        filter_name = "Extension"
        if filter['class'] in filter_names:
            filter_name = filter_names[filter['class']]

        if filter['class'] in filter_names.keys():
            filter_name = filter_names[filter['class']]
            pipeline[filter_name] = vals

    return pipeline

CHUNK_BASE = 16*1024    # Multiplier by which chunks are adjusted
CHUNK_MIN = 8*1024      # Soft lower limit (8k)
CHUNK_MAX = 1024*1024   # Hard upper limit (1M)

def guess_chunk(shape, maxshape, typesize):
    """ Guess an appropriate chunk layout for a dataset, given its shape and
    the size of each element in bytes.  Will allocate chunks only as large
    as MAX_SIZE.  Chunks are generally close to some power-of-2 fraction of
    each axis, slightly favoring bigger values for the last index.

    Undocumented and subject to change without warning.
    """
    # pylint: disable=unused-argument

    # For unlimited dimensions we have to guess 1024
    shape = tuple((x if x!=0 else 1024) for i, x in enumerate(shape))

    ndims = len(shape)
    if ndims == 0:
        raise ValueError("Chunks not allowed for scalar datasets.")

    chunks = np.array(shape, dtype='=f8')
    if not np.all(np.isfinite(chunks)):
        raise ValueError("Illegal value in chunk tuple")

    # Determine the optimal chunk size in bytes using a PyTables expression.
    # This is kept as a float.
    dset_size = np.product(chunks)*typesize
    target_size = CHUNK_BASE * (2**np.log10(dset_size/(1024.*1024)))

    if target_size > CHUNK_MAX:
        target_size = CHUNK_MAX
    elif target_size < CHUNK_MIN:
        target_size = CHUNK_MIN

    idx = 0
    while True:
        # Repeatedly loop over the axes, dividing them by 2.  Stop when:
        # 1a. We're smaller than the target chunk size, OR
        # 1b. We're within 50% of the target chunk size, AND
        #  2. The chunk is smaller than the maximum chunk size

        chunk_bytes = np.product(chunks)*typesize

        if (chunk_bytes < target_size or \
         abs(chunk_bytes-target_size)/target_size < 0.5) and \
         chunk_bytes < CHUNK_MAX:
            break

        if np.product(chunks) == 1:
            break  # Element size larger than CHUNK_MAX

        chunks[idx%ndims] = np.ceil(chunks[idx%ndims] / 2.0)
        idx += 1

    return tuple(int(x) for x in chunks)











from setuptools import setup
# run:
#   setup.py install
# or (if you'll be modifying the package):
#   setup.py develop
# To use a consistent encoding
# To upload to PyPI:
# python setup.py register sdist upload
#
# Tag the release in github!
#
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='h5pyd',
      version='0.2.7',
      description='h5py compatible client lib for HDF REST API',
      long_description=long_description,
      url='http://github.com/HDFGroup/h5pyd',
      author='John Readey',
      author_email='jreadey@hdfgrouup.org',
      license='BSD',
      packages=['h5pyd', 'h5pyd._hl', 'h5pyd._apps'],
      # requires=['h5py (>=2.5.0)', 'h5json>=1.0.2'],
      install_requires=['numpy >= 1.10.4', 'requests', 'six', 'pytz'],
      setup_requires=['pkgconfig', 'six'],
      zip_safe=False,
      entry_points={'console_scripts':
          ['hsinfo = h5pyd._apps.hsinfo:main', 
           'hsls = h5pyd._apps.hsls:main',
           'hstouch = h5pyd._apps.hstouch:main',
           'hsacl = h5pyd._apps.hsacl:main',
           'hsdel = h5pyd._apps.hsdel:main',
           'hsget = h5pyd._apps.hsget:main',
           'hsload = h5pyd._apps.hsload:main',
           'hsconfigure = h5pyd._apps.hsconfigure:main']
      },
)  



import os
import json
import six
import h5pyd
if __name__ == "__main__":
    from config import Config
else:
    from .config import Config

#
# input function that works with Python 2 or 3
#
def get_input(prompt):
    result = None
    if six.PY3:
        result = input(prompt)
    else:
        result = raw_input(prompt)
    return result

#
# Save configuration file
#
def saveConfig(username, password, endpoint, api_key):

    filepath = os.path.expanduser('~/.hscfg')
    print("Saving config file to: {}".format(filepath))
    with open(filepath, 'w') as file:
        file.write("# HDFCloud configuration file\n")
        if endpoint:
            file.write("hs_endpoint = {}\n".format(endpoint))
        else:
            file.write("hs_endpoint = \n")
        if username:
            file.write("hs_username = {}\n".format(username))
        else:
            file.write("hs_username = \n")
        if password:
            file.write("hs_password = {}\n".format(password))
        else:
            file.write("hs_password = \n")
        if api_key:
            file.write("hs_api_key = {}\n".format(api_key))
        else:
            file.write("hs_api_key = \n")
   
#
# Check to see if we can get a response from the server
#
def pingServer(username, password, endpoint, api_key):
    if not endpoint.startswith("http"):
        print("endpoint must start with 'http...'")
        return False

    try:
        info = h5pyd.getServerInfo(username=username, password=password, endpoint=endpoint, api_key=api_key)
        if 'state' not in info:
            print("unexpected response from server")
            return False
        state = info["state"]
        if state != "READY":
            print("Server is not ready, please try later")
            return False  
    except IOError as ioe:
        if ioe.errno == 401:
            print("Unauthorized (username/password or api key not valid)")
            return False
        elif ioe.errno == 403:
            print("forbidden (account not setup?)")
            return False
        elif ioe.errno:
            print("Unexpected error: {}".format(ioe.errno))
            return False
        else:
            print("Couldn't connect to server")
            return False
    except json.decoder.JSONDecodeError:
        print("Unexpected response from server")
        return False
    return True
#
# Main
#
def main():
    cfg = Config()

    hs_endpoint = cfg["hs_endpoint"]
    if not hs_endpoint:
        hs_endpoint = "None"
    hs_username = cfg["hs_username"]
    if not hs_username:
        hs_username = "None"
    hs_password = cfg["hs_password"]
    if not hs_password:
        hs_password = "None"
    hs_api_key = cfg["hs_api_key"]
    if not hs_api_key:
        hs_api_key = "None"

    done = False
    dirty = False

    while not done:
        print("Enter new values or accept defaults in brackets with Enter.")
        print("")
        new_endpoint = get_input("Server endpoint [{}]: ".format(hs_endpoint))
        if new_endpoint:
            print("Updated endpoint [{}]:".format(new_endpoint))
            hs_endpoint = new_endpoint
            dirty = True
         
        new_username = get_input("Username [{}]: ".format(hs_username))
        if new_username:
            print("Updated username: [{}]".format(new_username))
            hs_username = new_username
            dirty = True
        
        new_password = get_input("Password [{}]: ".format(hs_password))
        if new_password:
            print("updated password: [{}]".format(new_password))
            hs_password = new_password
            dirty = True

        new_api_key = get_input("API Key [{}]: ".format(hs_api_key))
        if new_api_key:
            print("updated api key: [{}]".format(new_api_key))
            hs_api_key = new_api_key
            dirty = True
        
        print("Testing connection...")
        ok = pingServer(hs_username, hs_password, hs_endpoint, hs_api_key)
        if ok:
            print("connection ok")
        if dirty:
            update = get_input("Save changes? (Y/N)")
            if update in ("Y", "y"):
                saveConfig(hs_username, hs_password, hs_endpoint, hs_api_key)
                break

        quit = get_input("Quit? (Y/N)")
        if quit in ("Y", "y"):
            break

if __name__ == "__main__":
    main()


##############################################################################
# Copyright by The HDF Group.                                                #
# All rights reserved.                                                       #
#                                                                            #
# This file is part of H5Serv (HDF5 REST Server) Service, Libraries and      #
# Utilities.  The full HDF5 REST Server copyright notice, including          #
# terms governing use, modification, and redistribution, is contained in     #
# the file COPYING, which can be found at the root of the source code        #
# distribution tree.  If you do not have access to this file, you may        #
# request a copy from help@hdfgroup.org.                                     #
##############################################################################
import json
import six
import config
if config.get("use_h5py"):
    import h5py
else:
    import h5pyd as h5py

from common import ut, TestCase


class TestObjRef(TestCase):


    def test_create(self):
        filename = self.getFileName("objref_test")
        print(filename)
        f = h5py.File(filename, 'w')
        self.assertTrue(f.id.id is not None)
        self.assertTrue('/' in f)
        r = f['/']
        is_h5serv = False
        if six.PY3:
            if isinstance(f.id.id, str) and not f.id.id.startswith("g-"):
                is_h5serv = True  # h5serv doesn't have support for objref datasets yet
        else:
            if isinstance(f.id.id, unicode) and not f.id.id.startswith(u"g-"):
                is_h5serv = True

        
        # create subgroup g1
        r.create_group('g1')
        self.assertTrue('g1' in r)
        g1 = r['g1']

        # create subgroup g1/g1.1
        g11 = g1.create_group('g1.1')

        # get ref to g1/g1.1
        g11_ref = g11.ref    
        self.assertTrue(isinstance(g11_ref, h5py.Reference))

        # create subgroup g2
        r.create_group('g2')
        self.assertEqual(len(r), 2)
        g2 = r['g2']
    
        # get ref to g1/g1.1 from g2
        g11ref = g2[g11_ref]
       
         
        # create subgroup /g1/g1.1/foo 
        g11ref.create_group("foo")   
        self.assertEqual(len(g11), 1)
        self.assertTrue("foo" in g11)
        
        # create datset /g2/d1
        d1 = g2.create_dataset('d1', (10,), dtype='i8')

        # get ref to d1
        d1_ref = d1.ref
        dt = h5py.special_dtype(ref=h5py.Reference)
        self.assertTrue(dt.metadata['ref'] is h5py.Reference)
        ref = h5py.check_dtype(ref=dt)
        self.assertEqual(ref, h5py.Reference)

        
        if not is_h5serv:
            # create dataset of ref types
            dset = g1.create_dataset('myrefs', (10,), dtype=dt)
            ref = h5py.check_dtype(ref=dset.dtype)
            self.assertEqual(ref, h5py.Reference)

            dset[0] = g11_ref
            dset[1] = d1_ref
           
            a_ref = dset[0]
            obj = f[a_ref]
            if not config.get("use_h5py"):
                self.assertEqual(obj.id.id, g11.id.id)  # ref to g1.1
            b_ref = dset[1]
            obj = f[b_ref]
            if not config.get("use_h5py"):
                self.assertEqual(obj.id.id, d1.id.id)  # ref to d1

            # try the same thing using attributes
            ref_values = [g11_ref, d1_ref]
            g1.attrs.create("a1", ref_values, dtype=dt)

            # read back the attribute
            attr =g1.attrs["a1"]
            self.assertEqual(attr.shape, (2,))
            ref = h5py.check_dtype(ref=attr.dtype)
            self.assertEqual(ref, h5py.Reference)
            a_ref = attr[0]
            obj = f[a_ref]
            if not config.get("use_h5py"):
                self.assertEqual(obj.id.id, g11.id.id)  # ref to g1.1
            b_ref = attr[1]
            obj = f[b_ref]
            if not config.get("use_h5py"):
                self.assertEqual(obj.id.id, d1.id.id)  # ref to d1

    

        f.close()

if __name__ == '__main__':
    ut.main()







##############################################################################
# Copyright by The HDF Group.                                                #
# All rights reserved.                                                       #
#                                                                            #
# This file is part of H5Serv (HDF5 REST Server) Service, Libraries and      #
# Utilities.  The full HDF5 REST Server copyright notice, including          #
# terms governing use, modification, and redistribution, is contained in     #
# the file COPYING, which can be found at the root of the source code        #
# distribution tree.  If you do not have access to this file, you may        #
# request a copy from help@hdfgroup.org.                                     #
##############################################################################
import config
if config.get("use_h5py"):
    import h5py
else:
    import h5pyd as h5py

from common import ut, TestCase
from datetime import datetime
import six


class TestGroup(TestCase):


    def test_cache(self):
        # create main test file
        filename = self.getFileName("create_group_cache")
        print("filename:", filename)
        f = h5py.File(filename, 'w', use_cache=True)
        self.assertTrue('/' in f)
        r = f['/'] 
        self.assertEqual(len(r), 0)
        self.assertTrue(isinstance(r, h5py.Group))
        self.assertTrue(r.name, '/')
        self.assertEqual(len(r.attrs.keys()), 0)
        self.assertFalse('g1' in r)
         
        g1 = r.create_group('g1')
        self.assertEqual(len(r), 1)
        file = g1.file
        f.close()

    def test_create(self):
        # create main test file
        filename = self.getFileName("create_group")
        print("filename:", filename)
        f = h5py.File(filename, 'w')
        is_hsds = False
        if isinstance(f.id.id, str) and f.id.id.startswith("g-"):
            is_hsds = True  # HSDS has different permission defaults
        self.assertTrue('/' in f)
        r = f['/'] 
        self.assertEqual(len(r), 0)
        self.assertTrue(isinstance(r, h5py.Group))
        self.assertTrue(r.name, '/')
        self.assertEqual(len(r.attrs.keys()), 0)
        self.assertFalse('g1' in r)

        r.create_group("g1")
        self.assertEqual(len(r), 1)
        self.assertTrue('g1' in r)

        g1 = r['g1']
        self.assertTrue(g1.id.id != r.id.id)
        self.assertEqual(g1.name, "/g1")

        r.create_group("g1/g1.1")
        g1_1 = r["g1/g1.1"]
        self.assertEqual(g1_1.name, "/g1/g1.1")
        self.assertEqual(len(r), 1)
        self.assertEqual(len(g1), 1)
           
        r.create_group('g2')
        self.assertEqual(len(r), 2)
        keys = []
        # iterate through keys
        for k in r:
            keys.append(k)

        self.assertEqual(len(keys), 2)
        self.assertTrue('g1' in keys)
        self.assertTrue('g2' in keys)

        self.assertTrue('g1' in r)
        self.assertTrue('/g1' in r)
        g1 = r.get('/g1')
        self.assertTrue(g1.id.id)
        self.assertEqual(g1.name, '/g1')

        g1_class = r.get('g1', getclass=True)
        self.assertEqual(g1_class, h5py.Group)

        # try creating a group that already exists
        try:
            r.create_group('g1')
            self.assertTrue(False)
        except ValueError:
            pass # expected

        r.create_group('g3')
        self.assertEqual(len(r), 3)
        del r['g3']
        self.assertEqual(len(r), 2)

        r.require_group('g4')
        self.assertEqual(len(r), 3)
        r.require_group('g2')
        self.assertEqual(len(r), 3)

        # create a hardlink
        tmp_grp = r.create_group("tmp")
        r['g1.1'] = tmp_grp

        # try to replace the link
        try:
            r['g1.1'] = g1_1
            self.assertTrue(False)  # shouldn't get here'
        except RuntimeError:
            pass # expected
        
        del r['tmp']
        self.assertEqual(len(r), 4)

        # create a softlink
        r['mysoftlink'] = h5py.SoftLink('/g1/g1.1')
        self.assertEqual(len(r), 5)
        self.assertEqual(len(g1), 1)
        self.assertEqual(len(g1_1), 0)

        slink = r['mysoftlink']
        self.assertEqual(slink.id, g1_1.id)

        # create a file that we'll link to
        link_target_filename = self.getFileName("link_target")
        g = h5py.File(link_target_filename, 'w')
        g.create_group("somepath")
        g.close()

        # create a external hardlink
        r['myexternallink'] = h5py.ExternalLink(link_target_filename, "somepath")
        # test getclass
        g1_class = r.get('g1', getclass=True)
        self.assertEqual(g1_class, h5py.Group)
        linkee_class = r.get('mysoftlink', getclass=True)
        self.assertEqual(linkee_class, h5py.Group)
        link_class = r.get('mysoftlink', getclass=True, getlink=True)
        self.assertEqual(link_class, h5py.SoftLink)
        softlink = r.get('mysoftlink', getlink=True)
        self.assertEqual(softlink.path, '/g1/g1.1')

        linkee_class = r.get('myexternallink', getclass=True)
        link_class = r.get('myexternallink', getclass=True, getlink=True)
        self.assertEqual(link_class, h5py.ExternalLink)
        external_link = r.get('myexternallink', getlink=True)
        self.assertEqual(external_link.path, 'somepath')
        external_link_filename = external_link.filename
        if config.get('use_h5py') or is_hsds:
            self.assertTrue(external_link_filename.find('link_target') > -1)
        else:
            self.assertTrue(external_link_filename.find('link_target') > -1)
            # HDF Server should be a DNS style name
            self.assertEqual(external_link_filename.find('/'), -1)

        links = r.items()
        got_external_link = False
        for link in links:
            title = link[0]
            obj = link[1]
            if title == 'myexternallink':
                self.assertTrue(obj is not None)            
                self.assertEqual(len(obj), 0)
                self.assertTrue(obj.file.filename != filename)
                got_external_link = True

        self.assertTrue(got_external_link)

        del r['mysoftlink']
        self.assertEqual(len(r), 5)

        del r['myexternallink']
        self.assertEqual(len(r), 4)

        # create group using nested path
        g2 = r['g2']
        r['g1/g1.3'] = g2
        self.assertEqual(len(r), 5)

        # try creating a link with a space in the name
        r["a space"] = g2
        self.assertEqual(len(r), 6)
         
        # Check group's last modified time
        if h5py.__name__ == "h5pyd":
            self.assertTrue(isinstance(g1.modified, datetime))
            #self.assertEqual(g1.modified.tzname(), six.u('UTC'))
         
        f.close()
        

if __name__ == '__main__':
    ut.main()




